package org.projectopen.timesheet;

/*
 * Copyright (C) 2010 ]project-open[ 
 *
 * This program is free software. You can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation;
 * either version 2 of the License, or (at your option)
 * any later version. This program is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * @author Frank Bergmann (frank.bergmann@project-open.com)
 * 
 */

import java.awt.TrayIcon;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.Reader;
import java.io.StringBufferInputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Hashtable;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

import com.sun.org.apache.xerces.internal.parsers.DOMParser;
import sun.misc.BASE64Encoder;

@SuppressWarnings({ "unchecked", "deprecation" })

public class RESTClient
{
    // hours logged today by the current user,
    // represented by a Hash from project_id to hours.
    // TODO: This may give us trouble if the app runs through the night...
	private static Hashtable loggedHoursToday = null;
    private static int userId = 0;
    
    public static int getUserId() {
    	if (0 == userId) {
    		ProjopObject user = getUserFromEmail(SystemTrayTest.getEmail());
    		userId = Integer.parseInt(user.get("user_id"));
    	}
		return userId;
	}

	private RESTClient()
    {
        super();
    }
         
    public static ProjopObject getObjectFromId(String objectType, int objectId) {
    	URL url;
    	Document dom = null;

    	ProjopObject o = new ProjopObject(objectType);
		o.setObjectId(objectId);

		try {
			url = new URL(SystemTrayTest.getServer() + "/intranet-rest/"+objectType+"/"+objectId+"?format=xml");
	    	dom = request(true, "GET", url, SystemTrayTest.getEmail(), SystemTrayTest.getPassword(), null);
		} catch (Exception e) {
			e.printStackTrace();
		}

		DomPrinter.walk(dom);		
		
		//get the root elememt
		Element docEle = dom.getDocumentElement();
		
		// Iterate through the list of <row> entries, 
		// adding the data to the list of projects.

		NodeList nl = docEle.getChildNodes();
		
//		NodeList nl1 = docEle.getElementsByTagName("row");
		if(nl != null && nl.getLength() > 0) {
			for(int i = 0 ; i < nl.getLength();i++) {				
				Node el = (Node)nl.item(i);
		        int type = el.getNodeType();
		        
		        // skip everything except for element nodes ("real" nodes).
		        if (type != Node.ELEMENT_NODE) { continue; }
		        
				String value = el.getTextContent();
				String name = el.getNodeName();
//				System.out.println(name+"="+value);

				o.set(name, value);
				
			}
		}

		return o;
   }

	public static ProjopObject getUserFromEmail(String email) {
    	List objects = getObjectsFromQuery("user", "email = '"+email+"'");
    	
    	if (objects == null) { return null; }
    	if (objects.isEmpty()) { return null; }    	
		return ((ProjopObject)objects.get(0));
   }
    
	public static List getObjectsFromQuery(String objectType, String query) {
    	URL url;
    	Document dom = null;
    	String encodedQuery = null;
    	List myObjects = new ArrayList();
    	
		try {
	    	encodedQuery = URLEncoder.encode(query, "UTF-8");
			// http://po34demo.project-open.net/intranet-rest/user?format=xml&query=username%20like%20'sys%'
			url = new URL(SystemTrayTest.getServer() + "/intranet-rest/"+objectType+"?format=xml&query="+encodedQuery);
	    	dom = request(true, "GET", url, SystemTrayTest.getEmail(), SystemTrayTest.getPassword(), null);
		} catch (Exception e) {
			e.printStackTrace();
		}

		if (dom == null) { return null; }
        DomPrinter.walk(dom);
		
		//get the root elememt
		Element docEle = dom.getDocumentElement();
		
		// Iterate through the list of <row> entries, 
		// adding the data to the list of projects.
		NodeList nl = docEle.getElementsByTagName("object_id");
		
		if(nl != null && nl.getLength() > 0) {
			for(int i = 0 ; i < nl.getLength();i++) {				
				Element el = (Element)nl.item(i);
				String objectIdString = el.getFirstChild().getNodeValue();
				int objectId = Integer.parseInt(objectIdString);
				ProjopObject o = RESTClient.getObjectFromId(objectType, objectId);
				myObjects.add(o);
			}
		}
		return myObjects;
   }

	public static Document request(boolean quiet, String method, URL url, String username, String password, InputStream body)
    {
        HttpURLConnection connection;
        int httpResponseCode = 0;
        
        if (!quiet)
        {
            System.out.println("[issuing request: " + method + " " + url + "]");
            System.out.println("[username: " + username + ", password: " + password + "]");
        }
        
        // Open the connection
		try {
			connection = (HttpURLConnection)url.openConnection();
	        connection.setRequestMethod(method);
		} catch (IOException e1) {
			SystemTrayTest.trayIconError(TrayIcon.MessageType.ERROR,"Connection Error", "Unable to connect to server "+SystemTrayTest.getServer());
			return null;
		}
        
        // Create and write Basic HTTP Authentication header
        BASE64Encoder encoder = new BASE64Encoder();
        String encodedCredential = encoder.encode( (username + ":" + password).getBytes() );
        connection.setRequestProperty("Authorization", "BASIC " + encodedCredential);
        
        // Write body if we're doing POST or PUT
        byte buffer[] = new byte[8192];
        int read = 0;
        if (body != null && method != "DELETE") {
    		try {
    			connection.setDoOutput(true);
    			OutputStream output = connection.getOutputStream();
    			while ((read = body.read(buffer)) != -1) { output.write(buffer, 0, read); }
    		} catch (IOException e2) {
    			SystemTrayTest.trayIconError(TrayIcon.MessageType.ERROR, "Connection Error", "Unable to write to connection to server "+SystemTrayTest.getServer());
    			return null;
    		}
        }
        
        // do request
        try {
			connection.connect();
	        httpResponseCode = connection.getResponseCode();
		} catch (IOException e2) {
			SystemTrayTest.trayIconError(TrayIcon.MessageType.ERROR, "Connection Error", "Unable to connect to server "+SystemTrayTest.getServer());
			return null;
		}
        
        /*
         * Check for connection errors. In particular, deal with
         * application specific errors like insufficient permissions
         * etc.
         */
        InputStream responseBodyStream = null;
        if (200 == httpResponseCode) {
        
        	// 200=Success.
        	try {
				responseBodyStream = connection.getInputStream();
			} catch (IOException e) {
				SystemTrayTest.trayIconError(TrayIcon.MessageType.ERROR, "Connection Error", "Unable to read from server "+SystemTrayTest.getServer());
				return null;
			}

        } else {
        	
        	// Any code != 200 means application error
        	try {
        		responseBodyStream = connection.getErrorStream();
        	
        		char charBuffer[] = new char[8192];
        		StringBuilder out = new StringBuilder();
        		Reader in = new InputStreamReader(responseBodyStream, "UTF-8");
        		do {
        			read = in.read(charBuffer, 0, buffer.length);
        			if (read > 0) { out.append(charBuffer, 0, read); }
        		} while (read >= 0);
    			System.out.println(out);
            	responseBodyStream = new StringBufferInputStream(out.toString());            	
        	
			} catch (IOException e) {
				SystemTrayTest.trayIconError(TrayIcon.MessageType.ERROR, "Connection Error", "Unable to read from server "+SystemTrayTest.getServer());
				return null;
			}

        	
        	// BufferedReader stdError = new BufferedReader(new InputStreamReader(responseBodyStream));
            // while (stdError.ready()) {
            //    String errorMsg = stdError.readLine();
            // 	System.out.println(errorMsg);
            // }
        	// The HTTPConnection returned a server error status.
        	// Let's inform the user about this error with a suitable message
        	// SystemTrayTest.trayIconError(TrayIcon.MessageType.ERROR, "Server Error", "The server returned HTTP Error "+responseCode);
        	// throw new HTTPException(responseCode);
        }
        
		DOMParser parser = new DOMParser();
		
		// Disable schema lookup, because www.w3c.com returns a 503 error
		try {
			parser.setFeature("http://xml.org/sax/features/validation", false);
			parser.setFeature("http://apache.org/xml/features/validation/schema", false);
			InputSource source = new InputSource (responseBodyStream);
			parser.parse(source);
		} catch (Exception e1) {
			SystemTrayTest.trayIconError(TrayIcon.MessageType.ERROR, "Application Error", "Unable to parse the reply from server "+SystemTrayTest.getServer());
			return null;
		}
		
	    Document dom = parser.getDocument();
	    
    	// The DOM contains an HTTP error message
	    // Inform the user about the error message and 
	    // return and empty DOM 
	    if (httpResponseCode != 200) {
			Element docEle = dom.getDocumentElement();
	    	DomPrinter.walk(docEle);
	    	httpError(httpResponseCode, url, docEle);
	    	dom = null;
	    }
	    
        return dom;
    }
    
    /*
     * The server has returned a reply like this one:
     * <error>
	 *	<http_status>403</http_status>
	 *	<http_status_message>Forbidden: The request is understood, but it has been refused.  An accompanying error message will explain why.</http_status_message>
	 *	<request>/intranet-reporting/view?format=xml&report%5fcode=rest%5fmy%5ftimesheet%5fprojects</request>
	 *	<message>The current user doesn't have the right to see this report.'</message>
	 * </error>
     */
	@SuppressWarnings("unused")
    public static void httpError(int httpResponseCode, URL url, Element docEle) {
		Integer httpStatus = ProjopObject.getIntValue(docEle, "http_status");
    	String httpStatusMessage = ProjopObject.getTextValue(docEle, "http_status_message");
    	String httpMessage = ProjopObject.getTextValue(docEle, "message");
    	String urlQuery = url.getQuery();
    	if (urlQuery == null) { urlQuery = ""; } else { urlQuery = "/"+urlQuery; }
    	httpMessage = httpMessage + 
    		"\non server " + SystemTrayTest.getServer() +
    		"\nat URL " + url.getPath() + urlQuery; 
		SystemTrayTest.trayIconError(TrayIcon.MessageType.ERROR, "Application Error", httpMessage);
    }
    
    public static Document projopGroups() {
    	URL url;
    	Document dom = null;
		try {
			url = new URL(SystemTrayTest.getServer() + "/intranet-reporting/view?format=xml&report_code=groups");
	    	dom = request(true, "GET", url, SystemTrayTest.getEmail(), SystemTrayTest.getPassword(), null);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return dom;
    }
    
	public static List projopMyTimesheetProjects() {
    	URL url;
    	Document dom = null;
    	List myProjects = new ArrayList();
    	
		try {
			url = new URL(SystemTrayTest.getServer() + "/intranet-reporting/view?format=xml&report_code=rest_my_timesheet_projects");
	    	dom = request(false, "GET", url, SystemTrayTest.getEmail(), SystemTrayTest.getPassword(), null);
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		//get the root elememt
		Element docEle = dom.getDocumentElement();
		
		// Iterate through the list of <row> entries, 
		// adding the data to the list of projects.
		NodeList nl = docEle.getElementsByTagName("row");
		if(nl != null && nl.getLength() > 0) {
			for(int i = 0 ; i < nl.getLength();i++) {				
				Element el = (Element)nl.item(i);
				ProjopProject p = ProjopProject.projectFromMyTimesheetProjectsXML(el);
				myProjects.add(p);
			}
		}
		return myProjects;
    }

    /*
     * Perform an "update" or "insert" on im_hours, translating into 
     * a POST operation either on the /intranet-rest/im_hour object 
     * type ("factory") for an insert, or on the im_hour/<hour_id>
     * concrete object for an update. 
     */
    public static void updateHours(ProjopHour h) {
    	Document dom = null;
    	boolean createNewObject = false;
    	String httpOperation = "POST";
    	
    	// get the hour_id of the hour. IF hour_id is null 
    	// THEN we need to perform an insert OTHERWISE an update.    	
    	String hourIdString = h.get("hour_id");
    	String hoursString = h.get("hours").trim();
    	Float hoursFloat = new Float(1.0);
    	try { hoursFloat = Float.parseFloat(hoursString); } catch (Exception e) {};
    	
    	// Convert into a string that can be passed directly to the
    	// ]po[ REST interface.
    	String xmlString = h.toXMLString();

    	// Determine the URL for the REST request.
    	// Depending on an "update" or "insert" operation, we have
    	// to choose the concrete object, or the object type ("factory"):
    	URL url = null;
		try {
			if (hourIdString == null) {
				url = new URL(SystemTrayTest.getServer() + "/intranet-rest/im_hour");
				createNewObject = true;
			} else {
				url = new URL(SystemTrayTest.getServer() + "/intranet-rest/im_hour/"+hourIdString);    		
				createNewObject = false;
			}
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
		
		// Perform the actual HTTP request to update/create the
		// hours in ]project-open[
		try {
			StringBuffer buf = new StringBuffer(xmlString);
			ByteArrayInputStream bis = new ByteArrayInputStream(buf.toString().getBytes("UTF-8"));
	    	dom = request(false, httpOperation, url, SystemTrayTest.getEmail(), SystemTrayTest.getPassword(), bis);
		} catch (Exception e) {
			e.printStackTrace();
		}

		// Check the returned value.
		// We'll need to extract the object_id and write
		// this to the ProjopHour object.
		if (createNewObject && dom != null) {
			// Creating a new object will return to us
			// the object_id (hour_id) in the XML reply
			Element docEle = dom.getDocumentElement();			
			DomPrinter.walk(docEle);
			
	        int type = docEle.getNodeType();
	        
	        // skip everything except for element nodes ("real" nodes).
	        if (type != Node.ELEMENT_NODE) { return; }
	        
			String value = docEle.getTextContent();
			String name = docEle.getNodeName();
			if ("object_id" == name) {
				h.set("hour_id",value);
			}
		}
		
		// ]project-open[ will delete the im_hours object if it's
		// "hours" field is empty of equal to 0.0.
		// This is a non-standard behaviour, because im_hours is not
		// a real object actually.
		// So we also need to mark our object in the loggedHoursToday
		// as deleted.
		if ("" == hoursString || 0.0 == hoursFloat) {
			Integer key = Integer.parseInt(h.get("project_id"));
			loggedHoursToday.remove(key);
		}
		
    	return;
    }
    
    /*
     * Write value back to 1) cache and 2) ]po[ server
     */
   	public static void setMyHours(String column, int projectId, Object value) {
   		
   		// create a new im_hour object and initialize with 
   		// the information necessary to create a new im_hours 
   		// entry in ]po[.
   		Integer key = new Integer(projectId);
   		ProjopHour h = (ProjopHour)(loggedHoursToday.get(key));

   		if (null == value) { value = ""; }
   		
   		if (h != null) {

   			// The hours object already exists
   			// Just update the element at the server.
   			h.set(column, value.toString());
   	   		
   	   		// write updated time to ]po[ server
   	   		updateHours(h);
   	   		return;

   		} else {
   		
   			// We have to create a new im_hour object
   			// on the ]po[ side, because we need the 
   			// objectId.
   			h = new ProjopHour();
   			h.set("project_id", projectId+"");
   	   		h.set("user_id", getUserId()+"");
   			h.set("hours", "0");
   	   		h.set("note", "");
   	   		h.set(column, value.toString());
   	   		
   	   		Date date = Calendar.getInstance().getTime();
   	   		DateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
   	   		String today = formatter.format(date);
   	   		
   	   		h.set("day", today.toString());	
   	   		
   	   		// write to Server
   	   		updateHours(h);
   			
   			loggedHoursToday.put(key, h);
   		}
   		System.out.println(h);
   		return;
	}
    
	public static ProjopHour projopMyHours(int projectId) {

    	URL url;
    	Document dom = null;

    	if (loggedHoursToday == null) {
			loggedHoursToday = new Hashtable();
			
			try {
				url = new URL(SystemTrayTest.getServer() + "/intranet-reporting/view?format=xml&report_code=rest_my_hours");
		    	dom = request(false, "GET", url, SystemTrayTest.getEmail(), SystemTrayTest.getPassword(), null);
			} catch (Exception e) {
				e.printStackTrace();
			}

			//get the root elememt
			Element docEle = dom.getDocumentElement();
			
			// Iterate through the list of <row> entries, 
			// adding the data to the list of projects.
			NodeList nl = docEle.getElementsByTagName("row");
			if(nl != null && nl.getLength() > 0) {
				for(int i = 0 ; i < nl.getLength();i++) {				
					Element el = (Element)nl.item(i);
					ProjopHour h = ProjopHour.hourFromMyHoursXML(el);
					Integer pId = Integer.parseInt(h.get("project_id"));
					loggedHoursToday.put(pId, h);
				}
			}
		}

    	// myHours now contains the list of all hours logged today
    	// by the current user.
    	Integer key = new Integer(projectId);
    	ProjopHour loggedHour = (ProjopHour)loggedHoursToday.get(key);
    	return loggedHour;
	}  
}
